@@ -114,6 +114,7 @@ CvSURFParams cvSURFParams(double threshold, int extended)
     CvSURFParams params;
     params.hessianThreshold = threshold;
     params.extended = extended;
+    params.upright = 0;
     params.nOctaves = 4;
     params.nOctaveLayers = 2;
     return params;
@@ -156,7 +157,7 @@ icvResizeHaarPattern( const int src[][5], CvSurfHF* dst, int n, int oldSize, int
  * Calculate the determinant and trace of the Hessian for a layer of the
  * scale-space pyramid
  */
-CV_INLINE void 
+CV_INLINE void
 icvCalcLayerDetAndTrace( const CvMat* sum, int size, int sampleStep, CvMat *det, CvMat *trace )
 {
     const int NX=3, NY=3, NXY=4;
@@ -184,7 +185,7 @@ icvCalcLayerDetAndTrace( const CvMat* sum, int size, int sampleStep, CvMat *det,
     /* Ignore pixels where some of the kernel is outside the image */
     margin = (size/2)/sampleStep;
 
-    for( i=0; i<samples_i; i++ )
+    for( i = 0; i < samples_i; i++ )
     {
         sum_ptr = sum->data.i + (i*sampleStep)*sum->cols;
         det_ptr = det->data.fl + (i+margin)*det->cols + margin;
@@ -220,15 +221,15 @@ icvCalcLayerDetAndTrace( const CvMat* sum, int size, int sampleStep, CvMat *det,
  * point contains the keypoint coordinates and scale to be modified
  *
  * Return value is 1 if interpolation was successful, 0 on failure.
- */   
-CV_INLINE int 
+ */
+CV_INLINE int
 icvInterpolateKeypoint( float N9[3][9], int dx, int dy, int ds, CvSURFPoint *point )
 {
     int solve_ok;
     float A[9], x[3], b[3];
     CvMat matA = cvMat(3, 3, CV_32F, A);
-    CvMat _x = cvMat(3, 1, CV_32F, x);                
-    CvMat _b = cvMat(3, 1, CV_32F, b);
+    CvMat _x   = cvMat(3, 1, CV_32F, x);
+    CvMat _b   = cvMat(3, 1, CV_32F, b);
 
     b[0] = -(N9[1][5]-N9[1][3])/2;  /* Negative 1st deriv with respect to x */
     b[1] = -(N9[1][7]-N9[1][1])/2;  /* Negative 1st deriv with respect to y */
@@ -275,9 +276,9 @@ icvFindMaximaInLayer( const CvMat *sum, const CvMat* mask_sum, const CvSURFParam
     CvSurfHF Dm;
     int i, j, size, margin, layer_rows, layer_cols;
     float *det_ptr, *trace_ptr;
- 
+
     size = sizes[layer];
-    
+
     /* The integral image 'sum' is one pixel bigger than the source image */
     layer_rows = (sum->rows-1)/sampleStep;
     layer_cols = (sum->cols-1)/sampleStep;
@@ -287,7 +288,7 @@ icvFindMaximaInLayer( const CvMat *sum, const CvMat* mask_sum, const CvSURFParam
 
     if( mask_sum )
        icvResizeHaarPattern( dm, &Dm, NM, 9, size, mask_sum->cols );
-    
+
     for( i = margin; i < layer_rows-margin; i++ )
     {
         det_ptr = dets[layer]->data.fl + i*dets[layer]->cols;
@@ -309,13 +310,13 @@ icvFindMaximaInLayer( const CvMat *sum, const CvMat* mask_sum, const CvSURFParam
                 const float *det1 = dets[layer-1]->data.fl + i*c + j;
                 const float *det2 = dets[layer]->data.fl   + i*c + j;
                 const float *det3 = dets[layer+1]->data.fl + i*c + j;
-                float N9[3][9] = { { det1[-c-1], det1[-c], det1[-c+1],          
+                float N9[3][9] = { { det1[-c-1], det1[-c], det1[-c+1],
                                      det1[-1]  , det1[0] , det1[1],
                                      det1[c-1] , det1[c] , det1[c+1]  },
-                                   { det2[-c-1], det2[-c], det2[-c+1],       
+                                   { det2[-c-1], det2[-c], det2[-c+1],
                                      det2[-1]  , det2[0] , det2[1],
                                      det2[c-1] , det2[c] , det2[c+1]  },
-                                   { det3[-c-1], det3[-c], det3[-c+1],       
+                                   { det3[-c-1], det3[-c], det3[-c+1],
                                      det3[-1]  , det3[0] , det3[1],
                                      det3[c-1] , det3[c] , det3[c+1]  } };
 
@@ -343,7 +344,7 @@ icvFindMaximaInLayer( const CvMat *sum, const CvMat* mask_sum, const CvSURFParam
                     double center_i = sum_i + (double)(size-1)/2;
                     double center_j = sum_j + (double)(size-1)/2;
 
-                    CvSURFPoint point = cvSURFPoint( cvPoint2D32f(center_j,center_i), 
+                    CvSURFPoint point = cvSURFPoint( cvPoint2D32f(center_j,center_i),
                                                      CV_SIGN(trace_ptr[j]), sizes[layer], 0, val0 );
 
                     /* Interpolate maxima location within the 3x3x3 neighbourhood  */
@@ -352,14 +353,14 @@ icvFindMaximaInLayer( const CvMat *sum, const CvMat* mask_sum, const CvSURFParam
 
                     /* Sometimes the interpolation step gives a negative size etc. */
                     if( interp_ok  )
-                    {   
+                    {
                         /*printf( "KeyPoint %f %f %d\n", point.pt.x, point.pt.y, point.size );*/
                     #ifdef HAVE_TBB
                         static tbb::mutex m;
                         tbb::mutex::scoped_lock lock(m);
-                    #endif                        
+                    #endif
                         cvSeqPush( points, &point );
-                    }    
+                    }
                 }
             }
         }
@@ -381,13 +382,13 @@ struct SURFBuildInvoker
         dets = _dets;
         traces = _traces;
     }
-   
+
     void operator()(const BlockedRange& range) const
-    { 
+    {
         for( int i=range.begin(); i<range.end(); i++ )
             icvCalcLayerDetAndTrace( sum, sizes[i], sampleSteps[i], dets[i], traces[i] );
     }
-    
+
     const CvMat *sum;
     const int *sizes;
     const int *sampleSteps;
@@ -422,7 +423,7 @@ struct SURFFindInvoker
             icvFindMaximaInLayer( sum, mask_sum, params, dets, traces, sizes, layer, 
                                   sampleSteps[layer], points );
         }
-    }    
+    }
 
     const CvMat *sum;
     const CvMat *mask_sum;
@@ -440,9 +441,9 @@ struct SURFFindInvoker
 
 
 /* Wavelet size at first layer of first octave. */ 
-const int HAAR_SIZE0 = 9;    
+const int HAAR_SIZE0 = 9;
 
-/* Wavelet size increment between layers. This should be an even number, 
+/* Wavelet size increment between layers. This should be an even number,
  such that the wavelet sizes in an octave are either all even or all odd.
  This ensures that when looking for the neighbours of a sample, the layers
  above and below are aligned correctly. */
@@ -455,20 +456,20 @@ static CvSeq* icvFastHessianDetector( const CvMat* sum, const CvMat* mask_sum,
     CvSeq* points = cvCreateSeq( 0, sizeof(CvSeq), sizeof(CvSURFPoint), storage );
 
     /* Sampling step along image x and y axes at first octave. This is doubled
-       for each additional octave. WARNING: Increasing this improves speed, 
+       for each additional octave. WARNING: Increasing this improves speed,
        however keypoint extraction becomes unreliable. */
-    const int SAMPLE_STEP0 = 1; 
+    const int SAMPLE_STEP0 = 1;
 
     int nTotalLayers = (params->nOctaveLayers+2)*params->nOctaves;
     int nMiddleLayers = params->nOctaveLayers*params->nOctaves;
 
-    CvMat** dets = (CvMat**)cvStackAlloc(nTotalLayers*sizeof(dets[0]));
-    CvMat** traces = (CvMat**)cvStackAlloc(nTotalLayers*sizeof(traces[0]));
-    int *sizes = (int*)cvStackAlloc(nTotalLayers*sizeof(sizes[0]));
-    int *sampleSteps = (int*)cvStackAlloc(nTotalLayers*sizeof(sampleSteps[0]));
-    int *middleIndices = (int*)cvStackAlloc(nMiddleLayers*sizeof(middleIndices[0]));
+    cv::AutoBuffer<CvMat*> dets(nTotalLayers);
+    cv::AutoBuffer<CvMat*> traces(nTotalLayers);
+    cv::AutoBuffer<int> sizes(nTotalLayers);
+    cv::AutoBuffer<int> sampleSteps(nTotalLayers);
+    cv::AutoBuffer<int> middleIndices(nMiddleLayers);
     int octave, layer, step, index, middleIndex;
- 
+
     /* Allocate space and calculate properties of each layer */
     index = 0;
     middleIndex = 0;
@@ -490,6 +491,7 @@ static CvSeq* icvFastHessianDetector( const CvMat* sum, const CvMat* mask_sum,
         step*=2;
     }
 
+#ifdef HAVE_TBB
     /* Calculate hessian determinant and trace samples in each layer*/
     cv::parallel_for( cv::BlockedRange(0, nTotalLayers),
                       cv::SURFBuildInvoker(sum,sizes,sampleSteps,dets,traces) );
@@ -498,6 +500,13 @@ static CvSeq* icvFastHessianDetector( const CvMat* sum, const CvMat* mask_sum,
     cv::parallel_for( cv::BlockedRange(0, nMiddleLayers),
                       cv::SURFFindInvoker(sum,mask_sum,params,dets,traces,sizes,
                                           sampleSteps,middleIndices,points) );
+#else
+    cv::SURFBuildInvoker(sum,sizes,sampleSteps,dets,traces)
+	    (cv::BlockedRange(0, nTotalLayers));
+
+    cv::SURFFindInvoker(sum,mask_sum,params,dets,traces,sizes, sampleSteps,middleIndices,points)
+	    ( cv::BlockedRange(0, nMiddleLayers) );
+#endif
 
     /* Clean-up */
     for( layer = 0; layer < nTotalLayers; layer++ )
@@ -514,17 +523,17 @@ namespace cv
 {
 
 /* Methods to free data allocated in SURFInvoker constructor */
-template<> inline void Ptr<float>::delete_obj(){ cvFree(&obj); }
-template<> inline void Ptr<CvPoint>::delete_obj(){ cvFree(&obj); }
+template<> inline void Ptr<float>::delete_obj()   { cvFree(&obj); }
+template<> inline void Ptr<CvPoint>::delete_obj() { cvFree(&obj); }
 
 struct SURFInvoker
 {
     enum { ORI_RADIUS = 6, ORI_WIN = 60, PATCH_SZ = 20 };
-    
-    static const int ORI_SEARCH_INC;
+
+    static const int   ORI_SEARCH_INC;
     static const float ORI_SIGMA;
     static const float DESC_SIGMA;
-    
+
     SURFInvoker( const CvSURFParams* _params,
                  CvSeq* _keypoints, CvSeq* _descriptors,
                  const CvMat* _img, const CvMat* _sum )
@@ -537,7 +546,7 @@ struct SURFInvoker
 
         /* Simple bound for number of grid points in circle of radius ORI_RADIUS */
         const int nOriSampleBound = (2*ORI_RADIUS+1)*(2*ORI_RADIUS+1);
-        
+
         /* Allocate arrays */
         apt = (CvPoint*)cvAlloc(nOriSampleBound*sizeof(CvPoint));
         aptw = (float*)cvAlloc(nOriSampleBound*sizeof(float));
@@ -567,16 +576,15 @@ struct SURFInvoker
                 DW[i*PATCH_SZ+j] = G_desc.at<float>(i,0) * G_desc.at<float>(j,0);
         }
     }
-    
+
     void operator()(const BlockedRange& range) const
     {
         /* X and Y gradient wavelet data */
         const int NX=2, NY=2;
         const int dx_s[NX][5] = {{0, 0, 2, 4, -1}, {2, 0, 4, 4, 1}};
         const int dy_s[NY][5] = {{0, 0, 4, 2, 1}, {0, 2, 4, 4, -1}};
-        
+
         const int descriptor_size = params->extended ? 128 : 64;
-        
         /* Optimisation is better using nOriSampleBound than nOriSamples for 
          array lengths.  Maybe because it is a constant known at compile time */
         const int nOriSampleBound =(2*ORI_RADIUS+1)*(2*ORI_RADIUS+1);
@@ -588,35 +596,28 @@ struct SURFInvoker
         CvMat matY = cvMat(1, nOriSampleBound, CV_32F, Y);
         CvMat _angle = cvMat(1, nOriSampleBound, CV_32F, angle);
         CvMat _patch = cvMat(PATCH_SZ+1, PATCH_SZ+1, CV_8U, PATCH);
-        
+
         int k, k1 = range.begin(), k2 = range.end();
         int maxSize = 0;
-        
         for( k = k1; k < k2; k++ )
         {
             maxSize = std::max(maxSize, ((CvSURFPoint*)cvGetSeqElem( keypoints, k ))->size);
         }
-        
         maxSize = cvCeil((PATCH_SZ+1)*maxSize*1.2f/9.0f);
         Ptr<CvMat> winbuf = cvCreateMat( 1, maxSize > 0 ? maxSize*maxSize : 1, CV_8U );
-        
         for( k = k1; k < k2; k++ )
         {
             const int* sum_ptr = sum->data.i;
             int sum_cols = sum->cols;
             int i, j, kk, x, y, nangle;
-            
             float* vec;
             CvSurfHF dx_t[NX], dy_t[NY];
-            
             CvSURFPoint* kp = (CvSURFPoint*)cvGetSeqElem( keypoints, k );
             int size = kp->size;
             CvPoint2D32f center = kp->pt;
-            
             /* The sampling intervals and wavelet sized for selecting an orientation
              and building the keypoint descriptor are defined relative to 's' */
             float s = (float)size*1.2f/9.0f;
-            
             /* To find the dominant orientation, the gradients in x and y are
              sampled in a circle of radius 6s using wavelets of size 4s.
              We ensure the gradient wavelet size is even to ensure the 
@@ -630,103 +631,134 @@ struct SURFInvoker
                 kp->size = -1;
                 continue;
             }
-            icvResizeHaarPattern( dx_s, dx_t, NX, 4, grad_wav_size, sum->cols );
-            icvResizeHaarPattern( dy_s, dy_t, NY, 4, grad_wav_size, sum->cols );
-            for( kk = 0, nangle = 0; kk < nOriSamples; kk++ )
+
+            float descriptor_dir = 90.f;
+            if (params->upright == 0)
             {
-                const int* ptr;
-                float vx, vy;
-                x = cvRound( center.x + apt[kk].x*s - (float)(grad_wav_size-1)/2 );
-                y = cvRound( center.y + apt[kk].y*s - (float)(grad_wav_size-1)/2 );
-                if( (unsigned)y >= (unsigned)(sum->rows - grad_wav_size) ||
-                   (unsigned)x >= (unsigned)(sum->cols - grad_wav_size) )
-                    continue;
-                ptr = sum_ptr + x + y*sum_cols;
-                vx = icvCalcHaarPattern( ptr, dx_t, 2 );
-                vy = icvCalcHaarPattern( ptr, dy_t, 2 );
-                X[nangle] = vx*aptw[kk]; Y[nangle] = vy*aptw[kk];
-                nangle++;
-            }
-            if ( nangle == 0 )
-            {
-                /* No gradient could be sampled because the keypoint is too
-                 * near too one or more of the sides of the image. As we
-                 * therefore cannot find a dominant direction, we skip this
-                 * keypoint and mark it for later deletion from the sequence. */
-                kp->size = -1;
-                continue;
-            }
-            matX.cols = matY.cols = _angle.cols = nangle;
-            cvCartToPolar( &matX, &matY, 0, &_angle, 1 );
-            
-            float bestx = 0, besty = 0, descriptor_mod = 0;
-            for( i = 0; i < 360; i += ORI_SEARCH_INC )
-            {
-                float sumx = 0, sumy = 0, temp_mod;
-                for( j = 0; j < nangle; j++ )
+                icvResizeHaarPattern( dx_s, dx_t, NX, 4, grad_wav_size, sum->cols );
+                icvResizeHaarPattern( dy_s, dy_t, NY, 4, grad_wav_size, sum->cols );
+                for( kk = 0, nangle = 0; kk < nOriSamples; kk++ )
                 {
-                    int d = std::abs(cvRound(angle[j]) - i);
-                    if( d < ORI_WIN/2 || d > 360-ORI_WIN/2 )
+                    const int* ptr;
+                    float vx, vy;
+                    x = cvRound( center.x + apt[kk].x*s - (float)(grad_wav_size-1)/2 );
+                    y = cvRound( center.y + apt[kk].y*s - (float)(grad_wav_size-1)/2 );
+                    if( (unsigned)y >= (unsigned)(sum->rows - grad_wav_size) ||
+                        (unsigned)x >= (unsigned)(sum->cols - grad_wav_size) )
+                        continue;
+                    ptr = sum_ptr + x + y*sum_cols;
+                    vx = icvCalcHaarPattern( ptr, dx_t, 2 );
+                    vy = icvCalcHaarPattern( ptr, dy_t, 2 );
+                    X[nangle] = vx*aptw[kk]; Y[nangle] = vy*aptw[kk];
+                    nangle++;
+                }
+                if ( nangle == 0 )
+                {
+                    /* No gradient could be sampled because the keypoint is too
+                    * near too one or more of the sides of the image. As we
+                    * therefore cannot find a dominant direction, we skip this
+                    * keypoint and mark it for later deletion from the sequence. */
+                    kp->size = -1;
+                    continue;
+                }
+                matX.cols = matY.cols = _angle.cols = nangle;
+                cvCartToPolar( &matX, &matY, 0, &_angle, 1 );
+
+                float bestx = 0, besty = 0, descriptor_mod = 0;
+                for( i = 0; i < 360; i += ORI_SEARCH_INC )
+                {
+                    float sumx = 0, sumy = 0, temp_mod;
+                    for( j = 0; j < nangle; j++ )
                     {
-                        sumx += X[j];
-                        sumy += Y[j];
+                        int d = std::abs(cvRound(angle[j]) - i);
+                        if( d < ORI_WIN/2 || d > 360-ORI_WIN/2 )
+                        {
+                            sumx += X[j];
+                            sumy += Y[j];
+                        }
+                    }
+                    temp_mod = sumx*sumx + sumy*sumy;
+                    if( temp_mod > descriptor_mod )
+                    {
+                        descriptor_mod = temp_mod;
+                        bestx = sumx;
+                        besty = sumy;
                     }
                 }
-                temp_mod = sumx*sumx + sumy*sumy;
-                if( temp_mod > descriptor_mod )
-                {
-                    descriptor_mod = temp_mod;
-                    bestx = sumx;
-                    besty = sumy;
-                }
+                descriptor_dir = cvFastArctan( besty, bestx );
             }
-            
-            float descriptor_dir = cvFastArctan( besty, bestx );
             kp->dir = descriptor_dir;
-            
             if( !descriptors )
                 continue;
             
-            descriptor_dir *= (float)(CV_PI/180);
-            
             /* Extract a window of pixels around the keypoint of size 20s */
             int win_size = (int)((PATCH_SZ+1)*s);
             CV_Assert( winbuf->cols >= win_size*win_size );
-            
             CvMat win = cvMat(win_size, win_size, CV_8U, winbuf->data.ptr);
-            float sin_dir = sin(descriptor_dir);
-            float cos_dir = cos(descriptor_dir) ;
-            
-            /* Subpixel interpolation version (slower). Subpixel not required since
-             the pixels will all get averaged when we scale down to 20 pixels */
-            /*  
-             float w[] = { cos_dir, sin_dir, center.x,
-             -sin_dir, cos_dir , center.y };
-             CvMat W = cvMat(2, 3, CV_32F, w);
-             cvGetQuadrangleSubPix( img, &win, &W );
-             */
-            
-            /* Nearest neighbour version (faster) */
-            float win_offset = -(float)(win_size-1)/2;
-            float start_x = center.x + win_offset*cos_dir + win_offset*sin_dir;
-            float start_y = center.y - win_offset*sin_dir + win_offset*cos_dir;
-            uchar* WIN = win.data.ptr;
-            for( i = 0; i < win_size; i++, start_x += sin_dir, start_y += cos_dir )
+
+            if (params->upright == 0)
             {
-                float pixel_x = start_x;
-                float pixel_y = start_y;
-                for( j = 0; j < win_size; j++, pixel_x += cos_dir, pixel_y -= sin_dir )
+            	descriptor_dir *= (float)(CV_PI/180);
+                float sin_dir = sin(descriptor_dir);
+                float cos_dir = cos(descriptor_dir);
+
+                /* Subpixel interpolation version (slower). Subpixel not required since
+                the pixels will all get averaged when we scale down to 20 pixels */
+                /*  
+                float w[] = { cos_dir, sin_dir, center.x,
+                -sin_dir, cos_dir , center.y };
+                CvMat W = cvMat(2, 3, CV_32F, w);
+                cvGetQuadrangleSubPix( img, &win, &W );
+                */
+
+                /* Nearest neighbour version (faster) */
+                float win_offset = -(float)(win_size-1)/2;
+                float start_x = center.x + win_offset*cos_dir + win_offset*sin_dir;
+                float start_y = center.y - win_offset*sin_dir + win_offset*cos_dir;
+                uchar* WIN = win.data.ptr;
+                for( i = 0; i < win_size; i++, start_x += sin_dir, start_y += cos_dir )
                 {
-                    int x = std::min(std::max(cvRound(pixel_x), 0), img->cols-1);
-                    int y = std::min(std::max(cvRound(pixel_y), 0), img->rows-1);
-                    WIN[i*win_size + j] = img->data.ptr[y*img->step + x];
+                    float pixel_x = start_x;
+                    float pixel_y = start_y;
+                    for( j = 0; j < win_size; j++, pixel_x += cos_dir, pixel_y -= sin_dir )
+                    {
+                        int x = std::min(std::max(cvRound(pixel_x), 0), img->cols-1);
+                        int y = std::min(std::max(cvRound(pixel_y), 0), img->rows-1);
+                        WIN[i*win_size + j] = img->data.ptr[y*img->step + x];
+                    }
                 }
             }
-            
+            else
+            {
+                /* extract rect - slightly optimized version of the code above
+                   TODO: find faster code, as this is simply an extract rect operation, 
+                         e.g. by using cvGetSubRect, problem is the border processing */
+                // descriptor_dir == 90 grad
+                // sin_dir == 1
+                // cos_dir == 0
+                
+                float win_offset = -(float)(win_size-1)/2;
+                int start_x = cvRound(center.x + win_offset);
+                int start_y = cvRound(center.y - win_offset);
+                uchar* WIN = win.data.ptr;
+                for( i = 0; i < win_size; i++, start_x++ )
+                {
+                    int pixel_x = start_x;
+                    int pixel_y = start_y;
+                    for( j=0; j<win_size; j++, pixel_y-- )
+                    {
+                        x = MAX( pixel_x, 0 );
+                        y = MAX( pixel_y, 0 );
+                        x = MIN( x, img->cols-1 );
+                        y = MIN( y, img->rows-1 );
+                        WIN[i*win_size + j] = img->data.ptr[y*img->step+x];
+                    }
+                }               
+            }
             /* Scale the window to size PATCH_SZ so each pixel's size is s. This
              makes calculating the gradients with wavelets of size 2s easy */
             cvResize( &win, &_patch, CV_INTER_AREA );
-            
+
             /* Calculate gradients in x and y with wavelets of size 2s */
             for( i = 0; i < PATCH_SZ; i++ )
                 for( j = 0; j < PATCH_SZ; j++ )
@@ -737,12 +769,12 @@ struct SURFInvoker
                     DX[i][j] = vx;
                     DY[i][j] = vy;
                 }
-            
+
             /* Construct the descriptor */
             vec = (float*)cvGetSeqElem( descriptors, k );
             for( kk = 0; kk < (int)(descriptors->elem_size/sizeof(vec[0])); kk++ )
                 vec[kk] = 0;
-            double square_mag = 0;       
+            double square_mag = 0;
             if( params->extended )
             {
                 /* 128-bin descriptor */
@@ -797,7 +829,7 @@ struct SURFInvoker
                         vec+=4;
                     }
             }
-            
+
             /* unit vector is essential for contrast invariance */
             vec = (float*)cvGetSeqElem( descriptors, k );
             double scale = 1./(sqrt(square_mag) + DBL_EPSILON);
@@ -805,24 +837,24 @@ struct SURFInvoker
                 vec[kk] = (float)(vec[kk]*scale);
         }
     }
-   
+
     /* Parameters */
     const CvSURFParams* params;
     const CvMat* img;
     const CvMat* sum;
     CvSeq* keypoints;
     CvSeq* descriptors;
-    
+
     /* Pre-calculated values */
     int nOriSamples;
-    cv::Ptr<CvPoint> apt; 
-    cv::Ptr<float> aptw;    
+    cv::Ptr<CvPoint> apt;
+    cv::Ptr<float> aptw;
     cv::Ptr<float> DW;
 };
-     
-const int SURFInvoker::ORI_SEARCH_INC = 5;  
-const float SURFInvoker::ORI_SIGMA = 2.5f;
-const float SURFInvoker::DESC_SIGMA = 3.3f;
+
+const int   SURFInvoker::ORI_SEARCH_INC = 5;
+const float SURFInvoker::ORI_SIGMA      = 2.5f;
+const float SURFInvoker::DESC_SIGMA     = 3.3f;
 }
 
 
@@ -830,7 +862,7 @@ CV_IMPL void
 cvExtractSURF( const CvArr* _img, const CvArr* _mask,
                CvSeq** _keypoints, CvSeq** _descriptors,
                CvMemStorage* storage, CvSURFParams params,
-			   int useProvidedKeyPts)
+               int useProvidedKeyPts)
 {
     CvMat *sum = 0, *mask1 = 0, *mask_sum = 0;
 
@@ -842,7 +874,7 @@ cvExtractSURF( const CvArr* _img, const CvArr* _mask,
     CvSeq *keypoints, *descriptors = 0;
     CvMat imghdr, *img = cvGetMat(_img, &imghdr);
     CvMat maskhdr, *mask = _mask ? cvGetMat(_mask, &maskhdr) : 0;
-    
+
     int descriptor_size = params.extended ? 128 : 64;
     const int descriptor_data_type = CV_32F;
     int i, N;
@@ -857,24 +889,24 @@ cvExtractSURF( const CvArr* _img, const CvArr* _mask,
 
     sum = cvCreateMat( img->rows+1, img->cols+1, CV_32SC1 );
     cvIntegral( img, sum );
-	
-	// Compute keypoints only if we are not asked for evaluating the descriptors are some given locations:
-	if (!useProvidedKeyPts)
-	{
-		if( mask )
-		{
-			mask1 = cvCreateMat( img->height, img->width, CV_8UC1 );
-			mask_sum = cvCreateMat( img->height+1, img->width+1, CV_32SC1 );
-			cvMinS( mask, 1, mask1 );
-			cvIntegral( mask1, mask_sum );
-		}
-		keypoints = icvFastHessianDetector( sum, mask_sum, storage, &params );
-	}
-	else
-	{
-		CV_Assert(useProvidedKeyPts && (_keypoints != 0) && (*_keypoints != 0));
-		keypoints = *_keypoints;
-	}
+
+    // Compute keypoints only if we are not asked for evaluating the descriptors are some given locations:
+    if (!useProvidedKeyPts)
+    {
+        if( mask )
+        {
+            mask1 = cvCreateMat( img->height, img->width, CV_8UC1 );
+            mask_sum = cvCreateMat( img->height+1, img->width+1, CV_32SC1 );
+            cvMinS( mask, 1, mask1 );
+            cvIntegral( mask1, mask_sum );
+        }
+        keypoints = icvFastHessianDetector( sum, mask_sum, storage, &params );
+    }
+    else
+    {
+        CV_Assert(useProvidedKeyPts && (_keypoints != 0) && (*_keypoints != 0));
+        keypoints = *_keypoints;
+    }
 
     N = keypoints->total;
     if( _descriptors )
@@ -884,10 +916,17 @@ cvExtractSURF( const CvArr* _img, const CvArr* _mask,
         cvSeqPushMulti( descriptors, 0, N );
     }
 
-     
-	if ( N > 0 )
-    cv::parallel_for(cv::BlockedRange(0, N),
+
+    if ( N > 0 )
+    {
+#ifdef HAVE_TBB
+        cv::parallel_for(cv::BlockedRange(0, N),
                      cv::SURFInvoker(&params, keypoints, descriptors, img, sum) );
+#else
+	    cv::SURFInvoker invoker(&params, keypoints, descriptors, img, sum);
+	    invoker(cv::BlockedRange(0, N));
+#endif
+    }
 
 
     /* remove keypoints that were marked for deletion */
@@ -922,21 +961,23 @@ SURF::SURF()
 {
     hessianThreshold = 100;
     extended = 1;
+    upright = 0;
     nOctaves = 4;
     nOctaveLayers = 2;
 }
 
-SURF::SURF(double _threshold, int _nOctaves, int _nOctaveLayers, bool _extended)
+SURF::SURF(double _threshold, int _nOctaves, int _nOctaveLayers, bool _extended, bool _upright)
 {
     hessianThreshold = _threshold;
     extended = _extended;
+    upright = _upright;
     nOctaves = _nOctaves;
     nOctaveLayers = _nOctaveLayers;
 }
 
 int SURF::descriptorSize() const { return extended ? 128 : 64; }
-    
-    
+
+
 static int getPointOctave(const CvSURFPoint& kpt, const CvSURFParams& params)
 {
     int octave = 0, layer = 0, best_octave = 0;
@@ -955,7 +996,7 @@ static int getPointOctave(const CvSURFPoint& kpt, const CvSURFParams& params)
         }
     return best_octave;
 }
-    
+
 
 void SURF::operator()(const Mat& img, const Mat& mask,
                       vector<KeyPoint>& keypoints) const
@@ -999,7 +1040,7 @@ void SURF::operator()(const Mat& img, const Mat& mask,
             kp.push_back(cvSURFPoint(kpt.pt, 1, cvRound(kpt.size), kpt.angle, kpt.response));
         }
     }
-    
+
     cvExtractSURF(&_img, pmask, &kp.seq, &d, storage,
         *(const CvSURFParams*)this, useProvidedKeypoints);
 
