@@ -5,14 +5,18 @@
 #include <cstdarg>
 #include <sstream>
 
+#include "opencv2/flann/miniflann.hpp"
 #include "opencv2/flann/dist.h"
 #include "opencv2/flann/index_testing.h"
-#include "opencv2/flann/logger.h"
+#include "opencv2/flann/params.h"
 #include "opencv2/flann/saving.h"
 #include "opencv2/flann/general.h"
+#include "opencv2/flann/dummy.h"
 
 // index types
 #include "opencv2/flann/all_indices.h"
 
+#include "opencv2/flann/flann_base.hpp"
+
 #endif
 
